TYPE
  LOGLEVEL : (CRITICAL, WARNING, INFO, DEBUG) := INFO;
END_TYPE

FUNCTION_BLOCK LOGGER
  VAR_INPUT
    TRIG : BOOL;
    MSG : STRING;
    LEVEL : LOGLEVEL := INFO;
  END_VAR
  VAR
    TRIG0 : BOOL;
  END_VAR

  IF TRIG AND NOT TRIG0 THEN
  {{
   LogMessage(GetFbVar(LEVEL),(char*)GetFbVar(MSG, .body),GetFbVar(MSG, .len));
  }}
  END_IF;
  TRIG0:=TRIG;
END_FUNCTION_BLOCK



FUNCTION_BLOCK python_eval
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(0, data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_poll
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(1,(PYTHON_EVAL*)(void*)data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_gear
  VAR_INPUT
    N : UINT;
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
    COUNTER : UINT;
    _TMP_ADD10_OUT : UINT;
    _TMP_EQ13_OUT : BOOL;
    _TMP_SEL15_OUT : UINT;
    _TMP_AND7_OUT : BOOL;
  END_VAR

  _TMP_ADD10_OUT := ADD(COUNTER, 1);
  _TMP_EQ13_OUT := EQ(N, _TMP_ADD10_OUT);
  _TMP_SEL15_OUT := SEL(_TMP_EQ13_OUT, _TMP_ADD10_OUT, 0);
  COUNTER := _TMP_SEL15_OUT;
  _TMP_AND7_OUT := AND(_TMP_EQ13_OUT, TRIG);
  py_eval(TRIG := _TMP_AND7_OUT, CODE := CODE);
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK


PROGRAM Arduino_MEGA_LD
  VAR
    I_Btn_S1_F : BOOL;
    I_Btn_S1_B : BOOL;
    I_Btn_S2_F : BOOL;
    I_Btn_S2_B : BOOL;
    I_Sensor1 : BOOL;
    I_Sensor2 : BOOL;
    I_Sensor3 : BOOL;
    I_Sensor4 : BOOL;
    I_Sensor5 : BOOL;
  END_VAR
  VAR
    I_Mill : BOOL := 1;
    I_Drill : BOOL := 1;
  END_VAR
  VAR
    Q_Slid1_F : BOOL;
    Q_Slid1_B : BOOL;
    Q_Slid2_F : BOOL;
    Q_Slid2_B : BOOL;
    Q_Conv1 : BOOL;
    Q_Conv2 : BOOL;
    Q_Conv3 : BOOL;
    Q_Conv4 : BOOL;
    Q_Mill : BOOL;
    Q_Drill : BOOL;
  END_VAR
  VAR
    TOF0 : TOF;
    TON0 : TON;
    TOF1 : TOF;
    TON1 : TON;
    TOF2 : TOF;
    TON2 : TON;
    TOF3 : TOF;
    TON3 : TON;
    TOF4 : TOF;
    TON4 : TON;
    F_TRIG1 : F_TRIG;
    F_TRIG2 : F_TRIG;
    F_TRIG3 : F_TRIG;
    F_TRIG4 : F_TRIG;
  END_VAR

  IF NOT(I_Sensor1) THEN
    Q_Conv1 := TRUE; (*set*)
  END_IF;
  TOF0(IN := NOT(I_Sensor2), PT := T#900ms);
  TON0(IN := TOF0.Q, PT := T#700ms);
  IF TON0.Q THEN
    Q_Conv1 := FALSE; (*reset*)
  END_IF;
  IF TON0.Q THEN
    Q_Conv2 := TRUE; (*set*)
  END_IF;
  IF TON0.Q THEN
    Q_Slid1_F := TRUE; (*set*)
  END_IF;
  IF I_Btn_S1_F THEN
    Q_Slid1_F := FALSE; (*reset*)
  END_IF;
  IF I_Btn_S1_F THEN
    Q_Slid1_B := TRUE; (*set*)
  END_IF;
  IF I_Btn_S1_B THEN
    Q_Slid1_B := FALSE; (*reset*)
  END_IF;
  F_TRIG1(CLK := I_Sensor3);
  TOF1(IN := I_Mill AND F_TRIG1.Q, PT := T#3000ms);
  IF TOF1.Q THEN
    Q_Mill := TRUE; (*set*)
  END_IF;
  IF TOF1.Q THEN
    Q_Conv2 := FALSE; (*reset*)
  END_IF;
  F_TRIG2(CLK := I_Sensor3);
  TON1(IN := TOF1.Q, PT := T#2900ms);
  TOF2(IN := NOT(I_Mill) AND F_TRIG2.Q OR TON1.Q, PT := T#1400ms);
  IF TOF2.Q THEN
    Q_Conv2 := TRUE; (*set*)
  END_IF;
  IF TOF2.Q THEN
    Q_Mill := FALSE; (*reset*)
  END_IF;
  IF TOF2.Q THEN
    Q_Conv3 := TRUE; (*set*)
  END_IF;
  TON2(IN := TOF2.Q, PT := T#1300ms);
  IF TON2.Q THEN
    Q_Conv2 := FALSE; (*reset*)
  END_IF;
  F_TRIG3(CLK := I_Sensor4);
  TOF3(IN := I_Drill AND F_TRIG3.Q, PT := T#3000ms);
  IF TOF3.Q THEN
    Q_Drill := TRUE; (*set*)
  END_IF;
  IF TOF3.Q THEN
    Q_Conv3 := FALSE; (*reset*)
  END_IF;
  F_TRIG4(CLK := I_Sensor4);
  TON3(IN := TOF3.Q, PT := T#2900ms);
  TOF4(IN := NOT(I_Drill) AND F_TRIG4.Q OR TON3.Q, PT := T#1500ms);
  IF TOF4.Q THEN
    Q_Drill := FALSE; (*reset*)
  END_IF;
  IF TOF4.Q THEN
    Q_Conv3 := TRUE; (*set*)
  END_IF;
  TON4(IN := TOF4.Q, PT := T#1400ms);
  IF TON4.Q THEN
    Q_Conv3 := FALSE; (*reset*)
  END_IF;
  IF TON4.Q THEN
    Q_Conv4 := TRUE; (*set*)
  END_IF;
  IF TON4.Q THEN
    Q_Slid2_F := TRUE; (*set*)
  END_IF;
  IF I_Btn_S2_F THEN
    Q_Slid2_F := FALSE; (*reset*)
  END_IF;
  IF I_Btn_S2_F THEN
    Q_Slid2_B := TRUE; (*set*)
  END_IF;
  IF I_Btn_S2_B THEN
    Q_Slid2_B := FALSE; (*reset*)
  END_IF;
  IF NOT(I_Sensor5) THEN
    Q_Conv4 := FALSE; (*reset*)
  END_IF;
END_PROGRAM


CONFIGURATION Config0

  RESOURCE Res0 ON PLC
    TASK task0(INTERVAL := T#20ms,PRIORITY := 0);
    PROGRAM instance0 WITH task0 : Arduino_MEGA_LD;
  END_RESOURCE
END_CONFIGURATION
