TYPE
  LOGLEVEL : (CRITICAL, WARNING, INFO, DEBUG) := INFO;
END_TYPE

FUNCTION_BLOCK LOGGER
  VAR_INPUT
    TRIG : BOOL;
    MSG : STRING;
    LEVEL : LOGLEVEL := INFO;
  END_VAR
  VAR
    TRIG0 : BOOL;
  END_VAR

  IF TRIG AND NOT TRIG0 THEN
  {{
   LogMessage(GetFbVar(LEVEL),(char*)GetFbVar(MSG, .body),GetFbVar(MSG, .len));
  }}
  END_IF;
  TRIG0:=TRIG;
END_FUNCTION_BLOCK



FUNCTION_BLOCK python_eval
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(0, data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_poll
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(1,(PYTHON_EVAL*)(void*)data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_gear
  VAR_INPUT
    N : UINT;
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
    COUNTER : UINT;
    ADD10_OUT : UINT;
    EQ13_OUT : BOOL;
    SEL15_OUT : UINT;
    AND7_OUT : BOOL;
  END_VAR

  ADD10_OUT := ADD(COUNTER, 1);
  EQ13_OUT := EQ(N, ADD10_OUT);
  SEL15_OUT := SEL(EQ13_OUT, ADD10_OUT, 0);
  COUNTER := SEL15_OUT;
  AND7_OUT := AND(EQ13_OUT, TRIG);
  py_eval(TRIG := AND7_OUT, CODE := CODE);
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK


PROGRAM SFC_test1
  VAR
    I_Sensor1 : BOOL;
    I_Sensor2 : BOOL;
    I_Sensor3 : BOOL;
    I_Sensor4 : BOOL;
    I_Sensor5 : BOOL;
    I_Btn_Left1 : BOOL;
    I_Btn_Left2 : BOOL;
    I_Btn_Right1 : BOOL;
    I_Btn_Right2 : BOOL;
    Q_Conv1 : BOOL;
    Q_Conv2 : BOOL;
    Q_Conv3 : BOOL;
    Q_Conv4 : BOOL;
    Q_Slider1_Forward : BOOL;
    Q_Slider1_Backward : BOOL;
    Q_Slider2_Forward : BOOL;
    Q_Slider2_Backward : BOOL;
    Q_Milling : BOOL;
    Q_Drilling : BOOL;
    I_Milling : BOOL := 1;
    I_Drilling : BOOL := 1;
  END_VAR
  VAR
    TON0 : TON;
    TON1 : TON;
    TOF0 : TOF;
    TOF1 : TOF;
    Start_timer_conv3 : BOOL := 0;
    Start_timer_mill : BOOL := 0;
    Start_timer_drill : BOOL := 0;
    TON2 : TON;
    TOF2 : TOF;
    TON3 : TON;
    TOF3 : TOF;
  END_VAR

  INITIAL_STEP Step0:
    COMPUTE_FUNCTION_BLOCKS(S);
  END_STEP

  ACTION COMPUTE_FUNCTION_BLOCKS:
    TOF0(IN := NOT I_Sensor2, PT := T#1500ms);
    TON0(IN := TOF0.Q, PT := T#1200ms);
    TOF2(IN := Start_timer_mill, PT := T#1200ms);
    TON2(IN := TOF2.Q, PT := T#1000ms);
    TOF1(IN := Start_timer_conv3, PT := T#1200ms);
    TON1(IN := TOF1.Q, PT := T#1000ms);
    TOF3(IN := Start_timer_drill, PT := T#1200ms);
    TON3(IN := TOF3.Q, PT := T#1000ms);
  END_ACTION

  TRANSITION FROM Step0 TO Step1
    := NOT I_Sensor1;
  END_TRANSITION

  STEP Step1:
    Q_Conv1(S);
  END_STEP

  TRANSITION FROM Step1 TO Step2
    := TON0.Q;
  END_TRANSITION

  STEP Step2:
    Q_Conv1(R);
  END_STEP

  TRANSITION FROM Step2 TO Step3
    := TRUE;
  END_TRANSITION

  STEP Step3:
    Q_Slider1_Forward(S);
  END_STEP

  TRANSITION FROM Step3 TO Step4
    := I_Btn_Left1;
  END_TRANSITION

  STEP Step4:
    Q_Slider1_Forward(R);
  END_STEP

  TRANSITION FROM Step4 TO Step5
    := TRUE;
  END_TRANSITION

  STEP Step5:
    Q_Conv2(S);
  END_STEP

  TRANSITION FROM Step5 TO Step6
    := NOT I_Sensor3;
  END_TRANSITION

  STEP Step6:
    Q_Conv2(R);
  END_STEP

  TRANSITION FROM Step6 TO Step7
    := TRUE;
  END_TRANSITION

  STEP Step7:
    Q_Slider1_Backward(S);
  END_STEP

  TRANSITION FROM Step7 TO Step8
    := I_Btn_Left2;
  END_TRANSITION

  STEP Step8:
    Q_Slider1_Backward(R);
  END_STEP

  TRANSITION FROM Step8 TO Step9
    := I_Milling;
  END_TRANSITION

  STEP Step9:
    Q_Milling(N);
    Start_timer_mill(N);
  END_STEP

  TRANSITION FROM Step9 TO Step10
    := TON2.Q;
  END_TRANSITION

  STEP Step10:
    Q_Conv2(S);
  END_STEP

  TRANSITION FROM Step10 TO Step11
    := TRUE;
  END_TRANSITION

  STEP Step11:
    Q_Conv3(S);
  END_STEP

  TRANSITION FROM Step11 TO Step12
    := NOT I_Sensor4;
  END_TRANSITION

  STEP Step12:
    Q_Conv3(R);
  END_STEP

  TRANSITION FROM Step12 TO Step13
    := TRUE;
  END_TRANSITION

  STEP Step13:
    Q_Conv2(R);
  END_STEP

  TRANSITION FROM Step13 TO Step14
    := I_Drilling;
  END_TRANSITION

  STEP Step14:
    Q_Drilling(N);
    Start_timer_drill(N);
  END_STEP

  TRANSITION FROM Step14 TO Step15
    := TON3.Q;
  END_TRANSITION

  STEP Step15:
    Q_Conv3(L, T#1500ms);
    Start_timer_conv3(N);
  END_STEP

  TRANSITION FROM Step15 TO Step16
    := TON1.Q;
  END_TRANSITION

  STEP Step16:
    Q_Slider2_Forward(S);
  END_STEP

  TRANSITION FROM Step16 TO Step17
    := I_Btn_Right1;
  END_TRANSITION

  STEP Step17:
    Q_Slider2_Forward(R);
  END_STEP

  TRANSITION FROM Step17 TO Step18
    := TRUE;
  END_TRANSITION

  STEP Step18:
    Q_Conv4(S);
  END_STEP

  TRANSITION FROM Step18 TO Step19
    := NOT I_Sensor5;
  END_TRANSITION

  STEP Step19:
    Q_Conv4(R);
  END_STEP

  TRANSITION FROM Step19 TO Step20
    := TRUE;
  END_TRANSITION

  STEP Step20:
    Q_Slider2_Backward(S);
  END_STEP

  TRANSITION FROM Step20 TO Step21
    := I_Btn_Right2;
  END_TRANSITION

  STEP Step21:
    Q_Slider2_Backward(R);
  END_STEP

  TRANSITION FROM Step21 TO Step0
    := TRUE;
  END_TRANSITION

  TRANSITION FROM Step13 TO Step15
    := NOT I_Drilling;
  END_TRANSITION

  TRANSITION FROM Step8 TO Step10
    := NOT I_Milling;
  END_TRANSITION

END_PROGRAM


CONFIGURATION Config0

  RESOURCE Res0 ON PLC
    TASK task0(INTERVAL := T#20ms,PRIORITY := 0);
    PROGRAM instance0 WITH task0 : SFC_test1;
  END_RESOURCE
END_CONFIGURATION
